/*
Copyright 2020 Rancher Labs.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by main. DO NOT EDIT.

package v1

import (
	"time"

	v1 "github.com/rancher/rio/pkg/apis/admin.rio.cattle.io/v1"
	scheme "github.com/rancher/rio/pkg/generated/clientset/versioned/scheme"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	types "k8s.io/apimachinery/pkg/types"
	watch "k8s.io/apimachinery/pkg/watch"
	rest "k8s.io/client-go/rest"
)

// ClusterDomainsGetter has a method to return a ClusterDomainInterface.
// A group's client should implement this interface.
type ClusterDomainsGetter interface {
	ClusterDomains() ClusterDomainInterface
}

// ClusterDomainInterface has methods to work with ClusterDomain resources.
type ClusterDomainInterface interface {
	Create(*v1.ClusterDomain) (*v1.ClusterDomain, error)
	Update(*v1.ClusterDomain) (*v1.ClusterDomain, error)
	UpdateStatus(*v1.ClusterDomain) (*v1.ClusterDomain, error)
	Delete(name string, options *metav1.DeleteOptions) error
	DeleteCollection(options *metav1.DeleteOptions, listOptions metav1.ListOptions) error
	Get(name string, options metav1.GetOptions) (*v1.ClusterDomain, error)
	List(opts metav1.ListOptions) (*v1.ClusterDomainList, error)
	Watch(opts metav1.ListOptions) (watch.Interface, error)
	Patch(name string, pt types.PatchType, data []byte, subresources ...string) (result *v1.ClusterDomain, err error)
	ClusterDomainExpansion
}

// clusterDomains implements ClusterDomainInterface
type clusterDomains struct {
	client rest.Interface
}

// newClusterDomains returns a ClusterDomains
func newClusterDomains(c *AdminV1Client) *clusterDomains {
	return &clusterDomains{
		client: c.RESTClient(),
	}
}

// Get takes name of the clusterDomain, and returns the corresponding clusterDomain object, and an error if there is any.
func (c *clusterDomains) Get(name string, options metav1.GetOptions) (result *v1.ClusterDomain, err error) {
	result = &v1.ClusterDomain{}
	err = c.client.Get().
		Resource("clusterdomains").
		Name(name).
		VersionedParams(&options, scheme.ParameterCodec).
		Do().
		Into(result)
	return
}

// List takes label and field selectors, and returns the list of ClusterDomains that match those selectors.
func (c *clusterDomains) List(opts metav1.ListOptions) (result *v1.ClusterDomainList, err error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	result = &v1.ClusterDomainList{}
	err = c.client.Get().
		Resource("clusterdomains").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Do().
		Into(result)
	return
}

// Watch returns a watch.Interface that watches the requested clusterDomains.
func (c *clusterDomains) Watch(opts metav1.ListOptions) (watch.Interface, error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	opts.Watch = true
	return c.client.Get().
		Resource("clusterdomains").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Watch()
}

// Create takes the representation of a clusterDomain and creates it.  Returns the server's representation of the clusterDomain, and an error, if there is any.
func (c *clusterDomains) Create(clusterDomain *v1.ClusterDomain) (result *v1.ClusterDomain, err error) {
	result = &v1.ClusterDomain{}
	err = c.client.Post().
		Resource("clusterdomains").
		Body(clusterDomain).
		Do().
		Into(result)
	return
}

// Update takes the representation of a clusterDomain and updates it. Returns the server's representation of the clusterDomain, and an error, if there is any.
func (c *clusterDomains) Update(clusterDomain *v1.ClusterDomain) (result *v1.ClusterDomain, err error) {
	result = &v1.ClusterDomain{}
	err = c.client.Put().
		Resource("clusterdomains").
		Name(clusterDomain.Name).
		Body(clusterDomain).
		Do().
		Into(result)
	return
}

// UpdateStatus was generated because the type contains a Status member.
// Add a +genclient:noStatus comment above the type to avoid generating UpdateStatus().

func (c *clusterDomains) UpdateStatus(clusterDomain *v1.ClusterDomain) (result *v1.ClusterDomain, err error) {
	result = &v1.ClusterDomain{}
	err = c.client.Put().
		Resource("clusterdomains").
		Name(clusterDomain.Name).
		SubResource("status").
		Body(clusterDomain).
		Do().
		Into(result)
	return
}

// Delete takes name of the clusterDomain and deletes it. Returns an error if one occurs.
func (c *clusterDomains) Delete(name string, options *metav1.DeleteOptions) error {
	return c.client.Delete().
		Resource("clusterdomains").
		Name(name).
		Body(options).
		Do().
		Error()
}

// DeleteCollection deletes a collection of objects.
func (c *clusterDomains) DeleteCollection(options *metav1.DeleteOptions, listOptions metav1.ListOptions) error {
	var timeout time.Duration
	if listOptions.TimeoutSeconds != nil {
		timeout = time.Duration(*listOptions.TimeoutSeconds) * time.Second
	}
	return c.client.Delete().
		Resource("clusterdomains").
		VersionedParams(&listOptions, scheme.ParameterCodec).
		Timeout(timeout).
		Body(options).
		Do().
		Error()
}

// Patch applies the patch and returns the patched clusterDomain.
func (c *clusterDomains) Patch(name string, pt types.PatchType, data []byte, subresources ...string) (result *v1.ClusterDomain, err error) {
	result = &v1.ClusterDomain{}
	err = c.client.Patch(pt).
		Resource("clusterdomains").
		SubResource(subresources...).
		Name(name).
		Body(data).
		Do().
		Into(result)
	return
}
