/*
Copyright 2020 Rancher Labs.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by main. DO NOT EDIT.

package v1

import (
	"time"

	v1 "github.com/rancher/rio/pkg/apis/admin.rio.cattle.io/v1"
	scheme "github.com/rancher/rio/pkg/generated/clientset/versioned/scheme"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	types "k8s.io/apimachinery/pkg/types"
	watch "k8s.io/apimachinery/pkg/watch"
	rest "k8s.io/client-go/rest"
)

// RioInfosGetter has a method to return a RioInfoInterface.
// A group's client should implement this interface.
type RioInfosGetter interface {
	RioInfos() RioInfoInterface
}

// RioInfoInterface has methods to work with RioInfo resources.
type RioInfoInterface interface {
	Create(*v1.RioInfo) (*v1.RioInfo, error)
	Update(*v1.RioInfo) (*v1.RioInfo, error)
	UpdateStatus(*v1.RioInfo) (*v1.RioInfo, error)
	Delete(name string, options *metav1.DeleteOptions) error
	DeleteCollection(options *metav1.DeleteOptions, listOptions metav1.ListOptions) error
	Get(name string, options metav1.GetOptions) (*v1.RioInfo, error)
	List(opts metav1.ListOptions) (*v1.RioInfoList, error)
	Watch(opts metav1.ListOptions) (watch.Interface, error)
	Patch(name string, pt types.PatchType, data []byte, subresources ...string) (result *v1.RioInfo, err error)
	RioInfoExpansion
}

// rioInfos implements RioInfoInterface
type rioInfos struct {
	client rest.Interface
}

// newRioInfos returns a RioInfos
func newRioInfos(c *AdminV1Client) *rioInfos {
	return &rioInfos{
		client: c.RESTClient(),
	}
}

// Get takes name of the rioInfo, and returns the corresponding rioInfo object, and an error if there is any.
func (c *rioInfos) Get(name string, options metav1.GetOptions) (result *v1.RioInfo, err error) {
	result = &v1.RioInfo{}
	err = c.client.Get().
		Resource("rioinfos").
		Name(name).
		VersionedParams(&options, scheme.ParameterCodec).
		Do().
		Into(result)
	return
}

// List takes label and field selectors, and returns the list of RioInfos that match those selectors.
func (c *rioInfos) List(opts metav1.ListOptions) (result *v1.RioInfoList, err error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	result = &v1.RioInfoList{}
	err = c.client.Get().
		Resource("rioinfos").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Do().
		Into(result)
	return
}

// Watch returns a watch.Interface that watches the requested rioInfos.
func (c *rioInfos) Watch(opts metav1.ListOptions) (watch.Interface, error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	opts.Watch = true
	return c.client.Get().
		Resource("rioinfos").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Watch()
}

// Create takes the representation of a rioInfo and creates it.  Returns the server's representation of the rioInfo, and an error, if there is any.
func (c *rioInfos) Create(rioInfo *v1.RioInfo) (result *v1.RioInfo, err error) {
	result = &v1.RioInfo{}
	err = c.client.Post().
		Resource("rioinfos").
		Body(rioInfo).
		Do().
		Into(result)
	return
}

// Update takes the representation of a rioInfo and updates it. Returns the server's representation of the rioInfo, and an error, if there is any.
func (c *rioInfos) Update(rioInfo *v1.RioInfo) (result *v1.RioInfo, err error) {
	result = &v1.RioInfo{}
	err = c.client.Put().
		Resource("rioinfos").
		Name(rioInfo.Name).
		Body(rioInfo).
		Do().
		Into(result)
	return
}

// UpdateStatus was generated because the type contains a Status member.
// Add a +genclient:noStatus comment above the type to avoid generating UpdateStatus().

func (c *rioInfos) UpdateStatus(rioInfo *v1.RioInfo) (result *v1.RioInfo, err error) {
	result = &v1.RioInfo{}
	err = c.client.Put().
		Resource("rioinfos").
		Name(rioInfo.Name).
		SubResource("status").
		Body(rioInfo).
		Do().
		Into(result)
	return
}

// Delete takes name of the rioInfo and deletes it. Returns an error if one occurs.
func (c *rioInfos) Delete(name string, options *metav1.DeleteOptions) error {
	return c.client.Delete().
		Resource("rioinfos").
		Name(name).
		Body(options).
		Do().
		Error()
}

// DeleteCollection deletes a collection of objects.
func (c *rioInfos) DeleteCollection(options *metav1.DeleteOptions, listOptions metav1.ListOptions) error {
	var timeout time.Duration
	if listOptions.TimeoutSeconds != nil {
		timeout = time.Duration(*listOptions.TimeoutSeconds) * time.Second
	}
	return c.client.Delete().
		Resource("rioinfos").
		VersionedParams(&listOptions, scheme.ParameterCodec).
		Timeout(timeout).
		Body(options).
		Do().
		Error()
}

// Patch applies the patch and returns the patched rioInfo.
func (c *rioInfos) Patch(name string, pt types.PatchType, data []byte, subresources ...string) (result *v1.RioInfo, err error) {
	result = &v1.RioInfo{}
	err = c.client.Patch(pt).
		Resource("rioinfos").
		SubResource(subresources...).
		Name(name).
		Body(data).
		Do().
		Into(result)
	return
}
