// Code generated by protoc-gen-gogo. DO NOT EDIT.
// source: github.com/solo-io/gloo/projects/gateway/api/v1/virtual_service.proto

package v1

import (
	bytes "bytes"
	fmt "fmt"
	math "math"

	_ "github.com/gogo/protobuf/gogoproto"
	proto "github.com/gogo/protobuf/proto"
	v1 "github.com/solo-io/gloo/projects/gloo/pkg/api/v1"
	matchers "github.com/solo-io/gloo/projects/gloo/pkg/api/v1/core/matchers"
	_ "github.com/solo-io/protoc-gen-ext/extproto"
	core "github.com/solo-io/solo-kit/pkg/api/v1/resources/core"
)

// Reference imports to suppress errors if they are not otherwise used.
var _ = proto.Marshal
var _ = fmt.Errorf
var _ = math.Inf

// This is a compile-time assertion to ensure that this generated file
// is compatible with the proto package it is being compiled against.
// A compilation error at this line likely means your copy of the
// proto package needs to be updated.
const _ = proto.GoGoProtoPackageIsVersion3 // please upgrade the proto package

//
//
// The **VirtualService** is the root routing object for the Gloo Gateway.
// A virtual service describes the set of routes to match for a set of domains.
//
// It defines:
// - a set of domains
// - the root set of routes for those domains
// - an optional SSL configuration for server TLS Termination
// - VirtualHostOptions that will apply configuration to all routes that live on the VirtualService.
//
// Domains must be unique across all virtual services within a gateway (i.e. no overlap between sets).
//
// VirtualServices can delegate routing behavior to the RouteTable resource by using the `delegateAction` on routes.
//
// An example configuration using two VirtualServices (one with TLS termination and one without) which share
// a RouteTable looks as follows:
//
// ```yaml
// # HTTP VirtualService:
// apiVersion: gateway.solo.io/v1
// kind: VirtualService
// metadata:
//   name: 'http'
//   namespace: 'usernamespace'
// spec:
//   virtualHost:
//     domains:
//     - '*.mydomain.com'
//     - 'mydomain.com'
//     routes:
//     - matchers:
//       - prefix: '/'
//       # delegate all traffic to the `shared-routes` RouteTable
//       delegateAction:
//         ref:
//           name: 'shared-routes'
//           namespace: 'usernamespace'
//
// ```
//
// ```yaml
// # HTTPS VirtualService:
// apiVersion: gateway.solo.io/v1
// kind: VirtualService
// metadata:
//   name: 'https'
//   namespace: 'usernamespace'
// spec:
//   virtualHost:
//     domains:
//     - '*.mydomain.com'
//     - 'mydomain.com'
//     routes:
//     - matchers:
//       - prefix: '/'
//       # delegate all traffic to the `shared-routes` RouteTable
//       delegateAction:
//         ref:
//           name: 'shared-routes'
//           namespace: 'usernamespace'
//   sslConfig:
//     secretRef:
//       name: gateway-tls
//       namespace: gloo-system
//
// ```
//
// ```yaml
// # the RouteTable shared by both VirtualServices:
// apiVersion: gateway.solo.io/v1
// kind: RouteTable
// metadata:
//   name: 'shared-routes'
//   namespace: 'usernamespace'
// spec:
//   routes:
//     - matchers:
//       - prefix: '/some-route'
//       routeAction:
//         single:
//           upstream:
//             name: 'some-upstream'
//      ...
// ```
//
// **Delegated Routes** are routes that use the `delegateAction` routing action. Delegated Routes obey the following
// constraints:
//
// - delegate routes must use `prefix` path matchers
// - delegated routes cannot specify header, query, or methods portion of the normal route matcher.
// - `routeOptions` configuration will be inherited from parent routes, but can be overridden by the child
//
type VirtualService struct {
	// The VirtualHost contains the
	// The list of HTTP routes define routing actions to be taken
	// for incoming HTTP requests whose host header matches
	// this virtual host. If the request matches more than one route in the list, the first route matched will be selected.
	// If the list of routes is empty, the virtual host will be ignored by Gloo.
	VirtualHost *VirtualHost `protobuf:"bytes,1,opt,name=virtual_host,json=virtualHost,proto3" json:"virtual_host,omitempty"`
	// If provided, the Gateway will serve TLS/SSL traffic for this set of routes
	SslConfig *v1.SslConfig `protobuf:"bytes,2,opt,name=ssl_config,json=sslConfig,proto3" json:"ssl_config,omitempty"`
	// Display only, optional descriptive name.
	// Unlike metadata.name, DisplayName can be any string
	// and can be changed after creating the resource.
	DisplayName string `protobuf:"bytes,3,opt,name=display_name,json=displayName,proto3" json:"display_name,omitempty"`
	// Status indicates the validation status of this resource.
	// Status is read-only by clients, and set by gloo during validation
	Status core.Status `protobuf:"bytes,6,opt,name=status,proto3" json:"status" testdiff:"ignore"`
	// Metadata contains the object metadata for this resource
	Metadata             core.Metadata `protobuf:"bytes,7,opt,name=metadata,proto3" json:"metadata"`
	XXX_NoUnkeyedLiteral struct{}      `json:"-"`
	XXX_unrecognized     []byte        `json:"-"`
	XXX_sizecache        int32         `json:"-"`
}

func (m *VirtualService) Reset()         { *m = VirtualService{} }
func (m *VirtualService) String() string { return proto.CompactTextString(m) }
func (*VirtualService) ProtoMessage()    {}
func (*VirtualService) Descriptor() ([]byte, []int) {
	return fileDescriptor_93fa9472926a2049, []int{0}
}
func (m *VirtualService) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_VirtualService.Unmarshal(m, b)
}
func (m *VirtualService) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_VirtualService.Marshal(b, m, deterministic)
}
func (m *VirtualService) XXX_Merge(src proto.Message) {
	xxx_messageInfo_VirtualService.Merge(m, src)
}
func (m *VirtualService) XXX_Size() int {
	return xxx_messageInfo_VirtualService.Size(m)
}
func (m *VirtualService) XXX_DiscardUnknown() {
	xxx_messageInfo_VirtualService.DiscardUnknown(m)
}

var xxx_messageInfo_VirtualService proto.InternalMessageInfo

func (m *VirtualService) GetVirtualHost() *VirtualHost {
	if m != nil {
		return m.VirtualHost
	}
	return nil
}

func (m *VirtualService) GetSslConfig() *v1.SslConfig {
	if m != nil {
		return m.SslConfig
	}
	return nil
}

func (m *VirtualService) GetDisplayName() string {
	if m != nil {
		return m.DisplayName
	}
	return ""
}

func (m *VirtualService) GetStatus() core.Status {
	if m != nil {
		return m.Status
	}
	return core.Status{}
}

func (m *VirtualService) GetMetadata() core.Metadata {
	if m != nil {
		return m.Metadata
	}
	return core.Metadata{}
}

//
//Virtual Hosts serve an ordered list of routes for a set of domains.
//
//An HTTP request is first matched to a virtual host based on its host header, then to a route within the virtual host.
//
//If a request is not matched to any virtual host or a route therein, the target proxy will reply with a 404.
//
//Unlike the [Gloo Virtual Host]({{< ref "/reference/api/github.com/solo-io/gloo/projects/gloo/api/v1/proxy.proto.sk.md" >}}/#virtualhost),
//Gateway* Virtual Hosts can **delegate** their routes to `RouteTables`.
//
type VirtualHost struct {
	// The list of domains (i.e.: matching the `Host` header of a request) that belong to this virtual host.
	// Note that the wildcard will not match the empty string. e.g. “*-bar.foo.com” will match “baz-bar.foo.com”
	// but not “-bar.foo.com”. Additionally, a special entry “*” is allowed which will match any host/authority header.
	// Only a single virtual host on a gateway can match on “*”. A domain must be unique across all
	// virtual hosts on a gateway or the config will be invalidated by Gloo
	// Domains on virtual hosts obey the same rules as [Envoy Virtual Hosts](https://github.com/envoyproxy/envoy/blob/master/api/envoy/api/v2/route/route.proto)
	Domains []string `protobuf:"bytes,2,rep,name=domains,proto3" json:"domains,omitempty"`
	// The list of HTTP routes define routing actions to be taken for incoming HTTP requests whose host header matches
	// this virtual host. If the request matches more than one route in the list, the first route matched will be selected.
	// If the list of routes is empty, the virtual host will be ignored by Gloo.
	Routes []*Route `protobuf:"bytes,3,rep,name=routes,proto3" json:"routes,omitempty"`
	// Virtual host options contain additional configuration to be applied to all traffic served by the Virtual Host.
	// Some configuration here can be overridden by Route Options.
	Options              *v1.VirtualHostOptions `protobuf:"bytes,4,opt,name=options,proto3" json:"options,omitempty"`
	XXX_NoUnkeyedLiteral struct{}               `json:"-"`
	XXX_unrecognized     []byte                 `json:"-"`
	XXX_sizecache        int32                  `json:"-"`
}

func (m *VirtualHost) Reset()         { *m = VirtualHost{} }
func (m *VirtualHost) String() string { return proto.CompactTextString(m) }
func (*VirtualHost) ProtoMessage()    {}
func (*VirtualHost) Descriptor() ([]byte, []int) {
	return fileDescriptor_93fa9472926a2049, []int{1}
}
func (m *VirtualHost) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_VirtualHost.Unmarshal(m, b)
}
func (m *VirtualHost) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_VirtualHost.Marshal(b, m, deterministic)
}
func (m *VirtualHost) XXX_Merge(src proto.Message) {
	xxx_messageInfo_VirtualHost.Merge(m, src)
}
func (m *VirtualHost) XXX_Size() int {
	return xxx_messageInfo_VirtualHost.Size(m)
}
func (m *VirtualHost) XXX_DiscardUnknown() {
	xxx_messageInfo_VirtualHost.DiscardUnknown(m)
}

var xxx_messageInfo_VirtualHost proto.InternalMessageInfo

func (m *VirtualHost) GetDomains() []string {
	if m != nil {
		return m.Domains
	}
	return nil
}

func (m *VirtualHost) GetRoutes() []*Route {
	if m != nil {
		return m.Routes
	}
	return nil
}

func (m *VirtualHost) GetOptions() *v1.VirtualHostOptions {
	if m != nil {
		return m.Options
	}
	return nil
}

//
// A route specifies how to match a request and what action to take when the request is matched.
//
// When a request matches on a route, the route can perform one of the following actions:
// - *Route* the request to a destination
// - Reply with a *Direct Response*
// - Send a *Redirect* response to the client
// - *Delegate* the action for the request to one or more top-level [`RouteTable`]({{< ref "/reference/api/github.com/solo-io/gloo/projects/gateway/api/v1/route_table.proto.sk.md" >}}) resources
// DelegateActions can be used to delegate the behavior for a set out routes with a given *prefix* to
// top-level `RouteTable` resources.
type Route struct {
	// Matchers contain parameters for matching requests (i.e., based on HTTP path, headers, etc.)
	// If empty, the route will match all requests (i.e, a single "/" path prefix matcher)
	// For delegated routes, the matcher must contain only a `prefix` path matcher and no other config
	Matchers []*matchers.Matcher `protobuf:"bytes,1,rep,name=matchers,proto3" json:"matchers,omitempty"`
	// The Route Action Defines what action the proxy should take when a request matches the route.
	//
	// Types that are valid to be assigned to Action:
	//	*Route_RouteAction
	//	*Route_RedirectAction
	//	*Route_DirectResponseAction
	//	*Route_DelegateAction
	Action isRoute_Action `protobuf_oneof:"action"`
	// Route Options extend the behavior of routes.
	// Route options include configuration such as retries, rate limiting, and request/response transformation.
	// RouteOption behavior will be inherited by delegated routes which do not specify their own `options`
	Options *v1.RouteOptions `protobuf:"bytes,6,opt,name=options,proto3" json:"options,omitempty"`
	// The name provides a convenience for users to be able to refer to a route by name.
	Name                 string   `protobuf:"bytes,7,opt,name=name,proto3" json:"name,omitempty"`
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *Route) Reset()         { *m = Route{} }
func (m *Route) String() string { return proto.CompactTextString(m) }
func (*Route) ProtoMessage()    {}
func (*Route) Descriptor() ([]byte, []int) {
	return fileDescriptor_93fa9472926a2049, []int{2}
}
func (m *Route) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_Route.Unmarshal(m, b)
}
func (m *Route) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_Route.Marshal(b, m, deterministic)
}
func (m *Route) XXX_Merge(src proto.Message) {
	xxx_messageInfo_Route.Merge(m, src)
}
func (m *Route) XXX_Size() int {
	return xxx_messageInfo_Route.Size(m)
}
func (m *Route) XXX_DiscardUnknown() {
	xxx_messageInfo_Route.DiscardUnknown(m)
}

var xxx_messageInfo_Route proto.InternalMessageInfo

type isRoute_Action interface {
	isRoute_Action()
	Equal(interface{}) bool
}

type Route_RouteAction struct {
	RouteAction *v1.RouteAction `protobuf:"bytes,2,opt,name=route_action,json=routeAction,proto3,oneof" json:"route_action,omitempty"`
}
type Route_RedirectAction struct {
	RedirectAction *v1.RedirectAction `protobuf:"bytes,3,opt,name=redirect_action,json=redirectAction,proto3,oneof" json:"redirect_action,omitempty"`
}
type Route_DirectResponseAction struct {
	DirectResponseAction *v1.DirectResponseAction `protobuf:"bytes,4,opt,name=direct_response_action,json=directResponseAction,proto3,oneof" json:"direct_response_action,omitempty"`
}
type Route_DelegateAction struct {
	DelegateAction *DelegateAction `protobuf:"bytes,5,opt,name=delegate_action,json=delegateAction,proto3,oneof" json:"delegate_action,omitempty"`
}

func (*Route_RouteAction) isRoute_Action()          {}
func (*Route_RedirectAction) isRoute_Action()       {}
func (*Route_DirectResponseAction) isRoute_Action() {}
func (*Route_DelegateAction) isRoute_Action()       {}

func (m *Route) GetAction() isRoute_Action {
	if m != nil {
		return m.Action
	}
	return nil
}

func (m *Route) GetMatchers() []*matchers.Matcher {
	if m != nil {
		return m.Matchers
	}
	return nil
}

func (m *Route) GetRouteAction() *v1.RouteAction {
	if x, ok := m.GetAction().(*Route_RouteAction); ok {
		return x.RouteAction
	}
	return nil
}

func (m *Route) GetRedirectAction() *v1.RedirectAction {
	if x, ok := m.GetAction().(*Route_RedirectAction); ok {
		return x.RedirectAction
	}
	return nil
}

func (m *Route) GetDirectResponseAction() *v1.DirectResponseAction {
	if x, ok := m.GetAction().(*Route_DirectResponseAction); ok {
		return x.DirectResponseAction
	}
	return nil
}

func (m *Route) GetDelegateAction() *DelegateAction {
	if x, ok := m.GetAction().(*Route_DelegateAction); ok {
		return x.DelegateAction
	}
	return nil
}

func (m *Route) GetOptions() *v1.RouteOptions {
	if m != nil {
		return m.Options
	}
	return nil
}

func (m *Route) GetName() string {
	if m != nil {
		return m.Name
	}
	return ""
}

// XXX_OneofWrappers is for the internal use of the proto package.
func (*Route) XXX_OneofWrappers() []interface{} {
	return []interface{}{
		(*Route_RouteAction)(nil),
		(*Route_RedirectAction)(nil),
		(*Route_DirectResponseAction)(nil),
		(*Route_DelegateAction)(nil),
	}
}

// DelegateActions are used to delegate routing decisions to Route Tables.
type DelegateAction struct {
	// The name of the Route Table to delegate to.
	// Deprecated: these fields have been added for backwards-compatibility. Please use the `single` field. If `name`
	// and/or `namespace` have been specified, Gloo will ignore `single` and `selector`.
	Name string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"` // Deprecated: Do not use.
	// The namespace of the Route Table to delegate to.
	// Deprecated: these fields have been added for backwards-compatibility. Please use the `single` field. If `name`
	// and/or `namespace` have been specified, Gloo will ignore `single` and `selector`.
	Namespace string `protobuf:"bytes,2,opt,name=namespace,proto3" json:"namespace,omitempty"` // Deprecated: Do not use.
	// Types that are valid to be assigned to DelegationType:
	//	*DelegateAction_Ref
	//	*DelegateAction_Selector
	DelegationType       isDelegateAction_DelegationType `protobuf_oneof:"delegation_type"`
	XXX_NoUnkeyedLiteral struct{}                        `json:"-"`
	XXX_unrecognized     []byte                          `json:"-"`
	XXX_sizecache        int32                           `json:"-"`
}

func (m *DelegateAction) Reset()         { *m = DelegateAction{} }
func (m *DelegateAction) String() string { return proto.CompactTextString(m) }
func (*DelegateAction) ProtoMessage()    {}
func (*DelegateAction) Descriptor() ([]byte, []int) {
	return fileDescriptor_93fa9472926a2049, []int{3}
}
func (m *DelegateAction) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_DelegateAction.Unmarshal(m, b)
}
func (m *DelegateAction) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_DelegateAction.Marshal(b, m, deterministic)
}
func (m *DelegateAction) XXX_Merge(src proto.Message) {
	xxx_messageInfo_DelegateAction.Merge(m, src)
}
func (m *DelegateAction) XXX_Size() int {
	return xxx_messageInfo_DelegateAction.Size(m)
}
func (m *DelegateAction) XXX_DiscardUnknown() {
	xxx_messageInfo_DelegateAction.DiscardUnknown(m)
}

var xxx_messageInfo_DelegateAction proto.InternalMessageInfo

type isDelegateAction_DelegationType interface {
	isDelegateAction_DelegationType()
	Equal(interface{}) bool
}

type DelegateAction_Ref struct {
	Ref *core.ResourceRef `protobuf:"bytes,3,opt,name=ref,proto3,oneof" json:"ref,omitempty"`
}
type DelegateAction_Selector struct {
	Selector *RouteTableSelector `protobuf:"bytes,4,opt,name=selector,proto3,oneof" json:"selector,omitempty"`
}

func (*DelegateAction_Ref) isDelegateAction_DelegationType()      {}
func (*DelegateAction_Selector) isDelegateAction_DelegationType() {}

func (m *DelegateAction) GetDelegationType() isDelegateAction_DelegationType {
	if m != nil {
		return m.DelegationType
	}
	return nil
}

// Deprecated: Do not use.
func (m *DelegateAction) GetName() string {
	if m != nil {
		return m.Name
	}
	return ""
}

// Deprecated: Do not use.
func (m *DelegateAction) GetNamespace() string {
	if m != nil {
		return m.Namespace
	}
	return ""
}

func (m *DelegateAction) GetRef() *core.ResourceRef {
	if x, ok := m.GetDelegationType().(*DelegateAction_Ref); ok {
		return x.Ref
	}
	return nil
}

func (m *DelegateAction) GetSelector() *RouteTableSelector {
	if x, ok := m.GetDelegationType().(*DelegateAction_Selector); ok {
		return x.Selector
	}
	return nil
}

// XXX_OneofWrappers is for the internal use of the proto package.
func (*DelegateAction) XXX_OneofWrappers() []interface{} {
	return []interface{}{
		(*DelegateAction_Ref)(nil),
		(*DelegateAction_Selector)(nil),
	}
}

// Select route tables for delegation by namespace, labels, or both.
type RouteTableSelector struct {
	// Delegate to Route Tables in these namespaces. If omitted, Gloo will only select Route Tables in the same namespace
	// as the resource (Virtual Service or Route Table) that owns this selector. The reserved value "*" can be used to
	// select Route Tables in all namespaces watched by Gloo.
	Namespaces []string `protobuf:"bytes,1,rep,name=namespaces,proto3" json:"namespaces,omitempty"`
	// Delegate to Route Tables whose labels match the ones specified here.
	Labels               map[string]string `protobuf:"bytes,2,rep,name=labels,proto3" json:"labels,omitempty" protobuf_key:"bytes,1,opt,name=key,proto3" protobuf_val:"bytes,2,opt,name=value,proto3"`
	XXX_NoUnkeyedLiteral struct{}          `json:"-"`
	XXX_unrecognized     []byte            `json:"-"`
	XXX_sizecache        int32             `json:"-"`
}

func (m *RouteTableSelector) Reset()         { *m = RouteTableSelector{} }
func (m *RouteTableSelector) String() string { return proto.CompactTextString(m) }
func (*RouteTableSelector) ProtoMessage()    {}
func (*RouteTableSelector) Descriptor() ([]byte, []int) {
	return fileDescriptor_93fa9472926a2049, []int{4}
}
func (m *RouteTableSelector) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_RouteTableSelector.Unmarshal(m, b)
}
func (m *RouteTableSelector) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_RouteTableSelector.Marshal(b, m, deterministic)
}
func (m *RouteTableSelector) XXX_Merge(src proto.Message) {
	xxx_messageInfo_RouteTableSelector.Merge(m, src)
}
func (m *RouteTableSelector) XXX_Size() int {
	return xxx_messageInfo_RouteTableSelector.Size(m)
}
func (m *RouteTableSelector) XXX_DiscardUnknown() {
	xxx_messageInfo_RouteTableSelector.DiscardUnknown(m)
}

var xxx_messageInfo_RouteTableSelector proto.InternalMessageInfo

func (m *RouteTableSelector) GetNamespaces() []string {
	if m != nil {
		return m.Namespaces
	}
	return nil
}

func (m *RouteTableSelector) GetLabels() map[string]string {
	if m != nil {
		return m.Labels
	}
	return nil
}

func init() {
	proto.RegisterType((*VirtualService)(nil), "gateway.solo.io.VirtualService")
	proto.RegisterType((*VirtualHost)(nil), "gateway.solo.io.VirtualHost")
	proto.RegisterType((*Route)(nil), "gateway.solo.io.Route")
	proto.RegisterType((*DelegateAction)(nil), "gateway.solo.io.DelegateAction")
	proto.RegisterType((*RouteTableSelector)(nil), "gateway.solo.io.RouteTableSelector")
	proto.RegisterMapType((map[string]string)(nil), "gateway.solo.io.RouteTableSelector.LabelsEntry")
}

func init() {
	proto.RegisterFile("github.com/solo-io/gloo/projects/gateway/api/v1/virtual_service.proto", fileDescriptor_93fa9472926a2049)
}

var fileDescriptor_93fa9472926a2049 = []byte{
	// 829 bytes of a gzipped FileDescriptorProto
	0x1f, 0x8b, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0xff, 0x8c, 0x55, 0x4f, 0x6f, 0xdb, 0x36,
	0x14, 0x8f, 0x2c, 0xc5, 0xb1, 0x9f, 0x83, 0xa4, 0x25, 0x8c, 0x4c, 0x31, 0x3a, 0xc7, 0x50, 0x30,
	0x34, 0x97, 0x4a, 0x58, 0x3b, 0x14, 0x9d, 0x81, 0xad, 0xa8, 0xd7, 0xa2, 0xc6, 0xb6, 0x6e, 0x00,
	0x33, 0xec, 0xd0, 0x8b, 0x41, 0x4b, 0xb4, 0xa2, 0x45, 0x16, 0x05, 0x92, 0xf6, 0xe2, 0x6b, 0xbf,
	0xc2, 0xbe, 0xc4, 0x3e, 0x42, 0x3f, 0xc2, 0x2e, 0xc3, 0xce, 0xbb, 0xf4, 0xb0, 0x6f, 0xd0, 0x01,
	0xbb, 0x0f, 0x22, 0x29, 0xd9, 0x72, 0x63, 0xa0, 0x27, 0x93, 0xef, 0xf7, 0x87, 0x4f, 0xef, 0x3d,
	0xd2, 0xf0, 0x22, 0x4e, 0xe4, 0xd5, 0x62, 0xea, 0x87, 0x6c, 0x1e, 0x08, 0x96, 0xb2, 0x07, 0x09,
	0x0b, 0xe2, 0x94, 0xb1, 0x20, 0xe7, 0xec, 0x17, 0x1a, 0x4a, 0x11, 0xc4, 0x44, 0xd2, 0x5f, 0xc9,
	0x2a, 0x20, 0x79, 0x12, 0x2c, 0x3f, 0x0f, 0x96, 0x09, 0x97, 0x0b, 0x92, 0x4e, 0x04, 0xe5, 0xcb,
	0x24, 0xa4, 0x7e, 0xce, 0x99, 0x64, 0xe8, 0xd8, 0xb0, 0xfc, 0xc2, 0xc3, 0x4f, 0x58, 0xaf, 0x1b,
	0xb3, 0x98, 0x29, 0x2c, 0x28, 0x56, 0x9a, 0xd6, 0x43, 0xf4, 0x46, 0xea, 0x20, 0xbd, 0x91, 0x26,
	0xd6, 0x57, 0xc7, 0x5e, 0x27, 0xb2, 0x3c, 0x61, 0x4e, 0x25, 0x89, 0x88, 0x24, 0x06, 0xbf, 0xb7,
	0x8d, 0x0b, 0x49, 0xe4, 0x42, 0x18, 0xf4, 0x74, 0x1b, 0xe5, 0x74, 0xb6, 0xcb, 0xb8, 0xdc, 0x1b,
	0xfc, 0x7c, 0xeb, 0x3b, 0x8b, 0x5d, 0xc9, 0x14, 0xa9, 0x21, 0x7d, 0xb6, 0x9b, 0x94, 0x73, 0x76,
	0xb3, 0x32, 0xb4, 0xfb, 0xbb, 0x69, 0x2c, 0x97, 0x09, 0xcb, 0xca, 0x7c, 0x1f, 0xef, 0x26, 0x86,
	0x8c, 0xd3, 0x60, 0x4e, 0x64, 0x78, 0x45, 0xb9, 0xa8, 0x16, 0x5a, 0xe7, 0xfd, 0xdd, 0x80, 0xa3,
	0x9f, 0x75, 0xe9, 0x2f, 0x75, 0xe5, 0xd1, 0x53, 0x38, 0x2c, 0x9b, 0x71, 0xc5, 0x84, 0x74, 0xad,
	0x81, 0x75, 0xd1, 0x79, 0x78, 0xcf, 0xdf, 0x6a, 0x85, 0x6f, 0x64, 0x63, 0x26, 0x24, 0xee, 0x2c,
	0xd7, 0x1b, 0xf4, 0x18, 0x40, 0x88, 0x74, 0x12, 0xb2, 0x6c, 0x96, 0xc4, 0x6e, 0x43, 0xc9, 0x3f,
	0xf1, 0x8b, 0x94, 0x2a, 0xed, 0xa5, 0x48, 0xbf, 0x51, 0x30, 0x6e, 0x8b, 0x72, 0x89, 0xee, 0xc3,
	0x61, 0x94, 0x88, 0x3c, 0x25, 0xab, 0x49, 0x46, 0xe6, 0xd4, 0xb5, 0x07, 0xd6, 0x45, 0x7b, 0xe4,
	0xbc, 0xfd, 0xcf, 0xb1, 0x70, 0xc7, 0x20, 0x3f, 0x90, 0x39, 0x45, 0xdf, 0x41, 0x53, 0x37, 0xcb,
	0x6d, 0x2a, 0xf3, 0xae, 0x5f, 0x7c, 0xe3, 0xda, 0x5c, 0x61, 0xa3, 0x4f, 0x0b, 0xe1, 0x1f, 0xef,
	0xce, 0xf6, 0xfe, 0x7d, 0x77, 0x76, 0x57, 0x52, 0x21, 0xa3, 0x64, 0x36, 0x1b, 0x7a, 0x49, 0x9c,
	0x31, 0x4e, 0x3d, 0x6c, 0x2c, 0xd0, 0x13, 0x68, 0x95, 0x93, 0xe1, 0x1e, 0x28, 0xbb, 0x93, 0xba,
	0xdd, 0x2b, 0x83, 0x8e, 0x9c, 0xc2, 0x0c, 0x57, 0xec, 0x61, 0xff, 0xcd, 0x7b, 0xc7, 0x81, 0xc6,
	0x52, 0xbc, 0x79, 0xef, 0x20, 0x74, 0x67, 0x6b, 0x82, 0x85, 0xf7, 0x9b, 0x05, 0x9d, 0x8d, 0x22,
	0x21, 0x17, 0x0e, 0x22, 0x36, 0x27, 0x49, 0x26, 0xdc, 0xc6, 0xc0, 0xbe, 0x68, 0xe3, 0x72, 0x8b,
	0x7c, 0x68, 0x72, 0xb6, 0x90, 0x54, 0xb8, 0xf6, 0xc0, 0x56, 0x19, 0x6c, 0x17, 0x1b, 0x17, 0x30,
	0x36, 0x2c, 0x34, 0x84, 0x03, 0xd3, 0x7e, 0xd7, 0x51, 0x29, 0x0f, 0xea, 0xe5, 0xdd, 0x38, 0xf5,
	0x47, 0xcd, 0xc3, 0xa5, 0xc0, 0xfb, 0xd3, 0x86, 0x7d, 0xe5, 0x86, 0x9e, 0x42, 0xab, 0x9c, 0x06,
	0xd7, 0x52, 0xe7, 0x9e, 0xfb, 0xd5, 0x78, 0xa8, 0x12, 0xd4, 0x4c, 0x5f, 0x69, 0x08, 0x57, 0x22,
	0xf4, 0x35, 0x1c, 0xaa, 0x84, 0x26, 0x24, 0x2c, 0xbc, 0x4d, 0xab, 0x4f, 0xeb, 0x32, 0x75, 0xd6,
	0x33, 0x45, 0x18, 0xef, 0xe1, 0x0e, 0x5f, 0x6f, 0xd1, 0x4b, 0x38, 0xe6, 0x34, 0x4a, 0x38, 0x0d,
	0x65, 0x69, 0x61, 0x97, 0xc3, 0x56, 0xb3, 0x30, 0xa4, 0xca, 0xe5, 0x88, 0xd7, 0x22, 0xe8, 0x35,
	0x9c, 0x18, 0x1b, 0x4e, 0x45, 0xce, 0x32, 0x51, 0xa5, 0xa4, 0xcb, 0xe3, 0xd5, 0xfd, 0x9e, 0x2b,
	0x2e, 0x36, 0xd4, 0xca, 0xb5, 0x1b, 0xdd, 0x12, 0x47, 0xdf, 0xc2, 0x71, 0x44, 0x53, 0x5a, 0x34,
	0xa4, 0x34, 0xdd, 0x57, 0xa6, 0x67, 0x1f, 0x34, 0xe9, 0xb9, 0xe1, 0xad, 0xf3, 0x8c, 0x6a, 0x11,
	0xf4, 0xc5, 0xba, 0x6f, 0x7a, 0x72, 0x7b, 0xb7, 0xd4, 0x6a, 0xbb, 0x63, 0x08, 0x81, 0xa3, 0xee,
	0x43, 0x31, 0x9d, 0x6d, 0xac, 0xd6, 0xa3, 0x16, 0x34, 0x75, 0x32, 0xde, 0x5f, 0x16, 0x1c, 0xd5,
	0x0f, 0x46, 0x27, 0x46, 0x60, 0xa9, 0x0b, 0xd4, 0x70, 0x2d, 0x2d, 0x42, 0x03, 0x68, 0x17, 0xbf,
	0x22, 0x27, 0x21, 0x55, 0xcd, 0xd2, 0xe0, 0x3a, 0x88, 0x1e, 0x80, 0xcd, 0xe9, 0xcc, 0x74, 0xe1,
	0xb4, 0x7e, 0x0f, 0x30, 0x15, 0x6c, 0xc1, 0x43, 0x8a, 0xe9, 0x6c, 0xbc, 0x87, 0x0b, 0x1e, 0x7a,
	0x06, 0x2d, 0x41, 0x53, 0x1a, 0x4a, 0xc6, 0x4d, 0xa5, 0xcf, 0x6f, 0x9f, 0xdc, 0x9f, 0xc8, 0x34,
	0xa5, 0x97, 0x86, 0x3a, 0xde, 0xc3, 0x95, 0x6c, 0x74, 0xb7, 0x2a, 0x6f, 0xc2, 0xb2, 0x89, 0x5c,
	0xe5, 0xd4, 0x7b, 0x6b, 0x01, 0xfa, 0x50, 0x85, 0xfa, 0x00, 0x55, 0xa2, 0x7a, 0x60, 0xdb, 0x78,
	0x23, 0x82, 0x5e, 0x42, 0x33, 0x25, 0x53, 0x9a, 0xea, 0xdb, 0xd5, 0x79, 0x18, 0x7c, 0x44, 0x2a,
	0xfe, 0xf7, 0x4a, 0xf1, 0x22, 0x93, 0x7c, 0x85, 0x8d, 0xbc, 0xf7, 0x25, 0x74, 0x36, 0xc2, 0xe8,
	0x0e, 0xd8, 0xd7, 0x74, 0xa5, 0x8b, 0x89, 0x8b, 0x25, 0xea, 0xc2, 0xfe, 0x92, 0xa4, 0x0b, 0x53,
	0x43, 0xac, 0x37, 0xc3, 0xc6, 0x13, 0x6b, 0xf4, 0x55, 0xf1, 0xe6, 0xfc, 0xfe, 0x4f, 0xdf, 0x7a,
	0xfd, 0xe8, 0xa3, 0xff, 0x00, 0xf3, 0xeb, 0xd8, 0x3c, 0xd5, 0xd3, 0xa6, 0x7a, 0x94, 0x1f, 0xfd,
	0x1f, 0x00, 0x00, 0xff, 0xff, 0x21, 0xb5, 0xee, 0xc2, 0x3e, 0x07, 0x00, 0x00,
}

func (this *VirtualService) Equal(that interface{}) bool {
	if that == nil {
		return this == nil
	}

	that1, ok := that.(*VirtualService)
	if !ok {
		that2, ok := that.(VirtualService)
		if ok {
			that1 = &that2
		} else {
			return false
		}
	}
	if that1 == nil {
		return this == nil
	} else if this == nil {
		return false
	}
	if !this.VirtualHost.Equal(that1.VirtualHost) {
		return false
	}
	if !this.SslConfig.Equal(that1.SslConfig) {
		return false
	}
	if this.DisplayName != that1.DisplayName {
		return false
	}
	if !this.Status.Equal(&that1.Status) {
		return false
	}
	if !this.Metadata.Equal(&that1.Metadata) {
		return false
	}
	if !bytes.Equal(this.XXX_unrecognized, that1.XXX_unrecognized) {
		return false
	}
	return true
}
func (this *VirtualHost) Equal(that interface{}) bool {
	if that == nil {
		return this == nil
	}

	that1, ok := that.(*VirtualHost)
	if !ok {
		that2, ok := that.(VirtualHost)
		if ok {
			that1 = &that2
		} else {
			return false
		}
	}
	if that1 == nil {
		return this == nil
	} else if this == nil {
		return false
	}
	if len(this.Domains) != len(that1.Domains) {
		return false
	}
	for i := range this.Domains {
		if this.Domains[i] != that1.Domains[i] {
			return false
		}
	}
	if len(this.Routes) != len(that1.Routes) {
		return false
	}
	for i := range this.Routes {
		if !this.Routes[i].Equal(that1.Routes[i]) {
			return false
		}
	}
	if !this.Options.Equal(that1.Options) {
		return false
	}
	if !bytes.Equal(this.XXX_unrecognized, that1.XXX_unrecognized) {
		return false
	}
	return true
}
func (this *Route) Equal(that interface{}) bool {
	if that == nil {
		return this == nil
	}

	that1, ok := that.(*Route)
	if !ok {
		that2, ok := that.(Route)
		if ok {
			that1 = &that2
		} else {
			return false
		}
	}
	if that1 == nil {
		return this == nil
	} else if this == nil {
		return false
	}
	if len(this.Matchers) != len(that1.Matchers) {
		return false
	}
	for i := range this.Matchers {
		if !this.Matchers[i].Equal(that1.Matchers[i]) {
			return false
		}
	}
	if that1.Action == nil {
		if this.Action != nil {
			return false
		}
	} else if this.Action == nil {
		return false
	} else if !this.Action.Equal(that1.Action) {
		return false
	}
	if !this.Options.Equal(that1.Options) {
		return false
	}
	if this.Name != that1.Name {
		return false
	}
	if !bytes.Equal(this.XXX_unrecognized, that1.XXX_unrecognized) {
		return false
	}
	return true
}
func (this *Route_RouteAction) Equal(that interface{}) bool {
	if that == nil {
		return this == nil
	}

	that1, ok := that.(*Route_RouteAction)
	if !ok {
		that2, ok := that.(Route_RouteAction)
		if ok {
			that1 = &that2
		} else {
			return false
		}
	}
	if that1 == nil {
		return this == nil
	} else if this == nil {
		return false
	}
	if !this.RouteAction.Equal(that1.RouteAction) {
		return false
	}
	return true
}
func (this *Route_RedirectAction) Equal(that interface{}) bool {
	if that == nil {
		return this == nil
	}

	that1, ok := that.(*Route_RedirectAction)
	if !ok {
		that2, ok := that.(Route_RedirectAction)
		if ok {
			that1 = &that2
		} else {
			return false
		}
	}
	if that1 == nil {
		return this == nil
	} else if this == nil {
		return false
	}
	if !this.RedirectAction.Equal(that1.RedirectAction) {
		return false
	}
	return true
}
func (this *Route_DirectResponseAction) Equal(that interface{}) bool {
	if that == nil {
		return this == nil
	}

	that1, ok := that.(*Route_DirectResponseAction)
	if !ok {
		that2, ok := that.(Route_DirectResponseAction)
		if ok {
			that1 = &that2
		} else {
			return false
		}
	}
	if that1 == nil {
		return this == nil
	} else if this == nil {
		return false
	}
	if !this.DirectResponseAction.Equal(that1.DirectResponseAction) {
		return false
	}
	return true
}
func (this *Route_DelegateAction) Equal(that interface{}) bool {
	if that == nil {
		return this == nil
	}

	that1, ok := that.(*Route_DelegateAction)
	if !ok {
		that2, ok := that.(Route_DelegateAction)
		if ok {
			that1 = &that2
		} else {
			return false
		}
	}
	if that1 == nil {
		return this == nil
	} else if this == nil {
		return false
	}
	if !this.DelegateAction.Equal(that1.DelegateAction) {
		return false
	}
	return true
}
func (this *DelegateAction) Equal(that interface{}) bool {
	if that == nil {
		return this == nil
	}

	that1, ok := that.(*DelegateAction)
	if !ok {
		that2, ok := that.(DelegateAction)
		if ok {
			that1 = &that2
		} else {
			return false
		}
	}
	if that1 == nil {
		return this == nil
	} else if this == nil {
		return false
	}
	if this.Name != that1.Name {
		return false
	}
	if this.Namespace != that1.Namespace {
		return false
	}
	if that1.DelegationType == nil {
		if this.DelegationType != nil {
			return false
		}
	} else if this.DelegationType == nil {
		return false
	} else if !this.DelegationType.Equal(that1.DelegationType) {
		return false
	}
	if !bytes.Equal(this.XXX_unrecognized, that1.XXX_unrecognized) {
		return false
	}
	return true
}
func (this *DelegateAction_Ref) Equal(that interface{}) bool {
	if that == nil {
		return this == nil
	}

	that1, ok := that.(*DelegateAction_Ref)
	if !ok {
		that2, ok := that.(DelegateAction_Ref)
		if ok {
			that1 = &that2
		} else {
			return false
		}
	}
	if that1 == nil {
		return this == nil
	} else if this == nil {
		return false
	}
	if !this.Ref.Equal(that1.Ref) {
		return false
	}
	return true
}
func (this *DelegateAction_Selector) Equal(that interface{}) bool {
	if that == nil {
		return this == nil
	}

	that1, ok := that.(*DelegateAction_Selector)
	if !ok {
		that2, ok := that.(DelegateAction_Selector)
		if ok {
			that1 = &that2
		} else {
			return false
		}
	}
	if that1 == nil {
		return this == nil
	} else if this == nil {
		return false
	}
	if !this.Selector.Equal(that1.Selector) {
		return false
	}
	return true
}
func (this *RouteTableSelector) Equal(that interface{}) bool {
	if that == nil {
		return this == nil
	}

	that1, ok := that.(*RouteTableSelector)
	if !ok {
		that2, ok := that.(RouteTableSelector)
		if ok {
			that1 = &that2
		} else {
			return false
		}
	}
	if that1 == nil {
		return this == nil
	} else if this == nil {
		return false
	}
	if len(this.Namespaces) != len(that1.Namespaces) {
		return false
	}
	for i := range this.Namespaces {
		if this.Namespaces[i] != that1.Namespaces[i] {
			return false
		}
	}
	if len(this.Labels) != len(that1.Labels) {
		return false
	}
	for i := range this.Labels {
		if this.Labels[i] != that1.Labels[i] {
			return false
		}
	}
	if !bytes.Equal(this.XXX_unrecognized, that1.XXX_unrecognized) {
		return false
	}
	return true
}
